/*******************************************************************************
**
**	File:		Cmsg.cpp
**
**	Description:	Functions for create Cyradis BMessages.
**
**	© Copyright 1997, Cyradis Corporation
**
*******************************************************************************/

#include "CMsg.h"


BMessenger *CreateMessenger(void) {
	status_t error;

	BMessenger *outlet;
	outlet = new BMessenger("application/x-vnd.XL-Toth",-1,&error);
	if (error != B_NO_ERROR)
		return NULL;
	else
		return outlet;
}




BMessage *CreateScene(const char *sig) {

	BMessage *message = new BMessage(C_NEWSCENE);
	message->AddString("Signature",sig);

	return message;
}


void SetId(BMessage *message, int32 id) {
	message->AddInt32("Id",id);
}


BMessage *CreateCamera(double PosX,double PosY, double PosZ, double LookX, double LookY, double LookZ, double Roll, double Lens) {
	BMessage *message = new BMessage(C_CAMERA);

	message->AddDouble("PosX",PosX);
	message->AddDouble("PosY",PosY);
	message->AddDouble("PosZ",PosZ);

	message->AddDouble("LookX",LookX);
	message->AddDouble("LookY",LookY);
	message->AddDouble("LookZ",LookZ);

	message->AddDouble("Roll",Roll);
	message->AddDouble("Lens",Lens);
	
	return message;
}



BMessage *CreateRender(int32 height, int32 width, bool antialiasing,int32 linestart, int32 lineend) {

	BMessage *message = new BMessage(C_RENDER);

	message->AddInt32("Height",height);
	message->AddInt32("Width",width);
	message->AddBool("AntiAliasing",antialiasing);
	message->AddInt32("LineStart",linestart);
	message->AddInt32("LineEnd",lineend);

	return message;
}

void SetObjectColor(BMessage *obj,rgb_color &color) {
	obj->AddInt16("ColorRed",color.red);
	obj->AddInt16("ColorGreen",color.green);
	obj->AddInt16("ColorBlue",color.blue);
}

void SetObjectAmbient(BMessage *obj,double ambient) {
	obj->AddDouble("Ambient",ambient);
}
void SetObjectDiffuse(BMessage *obj,double diffuse) {
	obj->AddDouble("Diffuse",diffuse);
}
void SetObjectReflection(BMessage *obj,double reflection) {
	obj->AddDouble("Reflection",reflection);
}
void SetObjectTransparency(BMessage *obj,double transparency) {
	obj->AddDouble("Transparency",transparency);
}
void SetObjectIor(BMessage *obj,double ior) {
	obj->AddDouble("Ior",ior);
}
void SetObjectSpecular(BMessage *obj,double specular) {
	obj->AddDouble("Specular",specular);
}
void SetObjectSpecularSize(BMessage *obj,double specsize){
	obj->AddDouble("SpecularSize",specsize);
}


BMessage *CreateSphere(double X, double Y, double Z, double Radius) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_SPHERE);

	// Add the sphere information
	message->AddDouble("X",X);
	message->AddDouble("Y",Y);
	message->AddDouble("Z",Z);
	message->AddDouble("Radius",Radius);
	return message;
}


BMessage *CreatePlane(double A, double B, double C, double D) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_PLANE);

	// Add the plane information
	message->AddDouble("A",A);
	message->AddDouble("B",B);
	message->AddDouble("C",C);
	message->AddDouble("D",D);
	return message;
}

BMessage *CreateDisc(double x1,double y1,double z1, double x2, double y2, double z2, double radiusmin, double radiusmax) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_DISC);

	// Add the Disc information
	message->AddDouble("PosX",x1);
	message->AddDouble("PosY",y1);
	message->AddDouble("PosZ",z1);
	message->AddDouble("NormX",x2);
	message->AddDouble("NormY",y2);
	message->AddDouble("NormZ",z2);
	message->AddDouble("RadiusMin",radiusmin);
	message->AddDouble("RadiusMax",radiusmax);
	return message;
}


BMessage *CreateTriangle(double ax,double ay,double az,double bx,double by,double bz,double cx,double cy,double cz) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_TRIANGLE);

	// Add the Triangle information
	message->AddDouble("Ax",ax);
	message->AddDouble("Ay",ay);
	message->AddDouble("Az",az);
	message->AddDouble("Bx",bx);
	message->AddDouble("By",by);
	message->AddDouble("Bz",bz);
	message->AddDouble("Cx",cx);
	message->AddDouble("Cy",cy);
	message->AddDouble("Cz",cz);
	return message;
}

BMessage *CreateBox(double ax,double ay,double az,double bx,double by,double bz) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_BOX);
	// Add the Box information
	message->AddDouble("Ax",ax);
	message->AddDouble("Ay",ay);
	message->AddDouble("Az",az);
	message->AddDouble("Bx",bx);
	message->AddDouble("By",by);
	message->AddDouble("Bz",bz);

	return message;
}

BMessage *CreateCone(double X1, double Y1, double Z1, double Radius1, double X2, double Y2, double Z2, double Radius2) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_CONE);

	// Add the cone information
	message->AddDouble("X1",X1);
	message->AddDouble("Y1",Y1);
	message->AddDouble("Z1",Z1);
	message->AddDouble("Radius1",Radius1);
	message->AddDouble("X2",X2);
	message->AddDouble("Y2",Y2);
	message->AddDouble("Z2",Z2);
	message->AddDouble("Radius2",Radius2);
	return message;
}


BMessage *CreateCylinder(double X1, double Y1, double Z1, double X2, double Y2, double Z2, double Radius) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_CYLINDER);

	// Add the cylinder information
	message->AddDouble("X1",X1);
	message->AddDouble("Y1",Y1);
	message->AddDouble("Z1",Z1);
	message->AddDouble("X2",X2);
	message->AddDouble("Y2",Y2);
	message->AddDouble("Z2",Z2);
	message->AddDouble("Radius",Radius);
	return message;
}

void SetConeOpen(BMessage *cone) {
	cone->AddBool("Open",true);
}

void SetCylinderOpen(BMessage *cylinder) {
	cylinder->AddBool("Open",true);
}

BMessage *CreateQuadric(double A, double B, double C, double D, double E, double F, double G, double H, double I, double J) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_QUADRIC);

	// Add the quadric information
	message->AddDouble("A",A);
	message->AddDouble("B",B);
	message->AddDouble("C",C);
	message->AddDouble("D",D);
	message->AddDouble("E",E);
	message->AddDouble("F",F);
	message->AddDouble("G",G);
	message->AddDouble("H",H);
	message->AddDouble("I",I);
	message->AddDouble("J",J);
	return message;
}


BMessage *CreateDifference(const BMessage *object1, const BMessage *object2) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_DIFFERENCE);

	message->AddMessage("Op1",object1);
	message->AddMessage("Op2",object2);
	return message;
}

BMessage *CreateUnion(const BMessage *object1, const BMessage *object2) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_UNION);

	message->AddMessage("Op1",object1);
	message->AddMessage("Op2",object2);
	return message;
}

BMessage *CreateMerge(const BMessage *object1, const BMessage *object2) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_MERGE);

	message->AddMessage("Op1",object1);
	message->AddMessage("Op2",object2);
	return message;
}

BMessage *CreateClip(const BMessage *object1, const BMessage *object2) {
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_CLIP);

	message->AddMessage("Op1",object1);
	message->AddMessage("Op2",object2);
	return message;
}

BMessage *CreateIntersection(const BMessage *object1, const BMessage *object2){
	BMessage *message = new BMessage(C_NEWOBJECT);
	message->AddInt32("Type",C_INTERSECTION);

	message->AddMessage("Op1",object1);
	message->AddMessage("Op2",object2);
	return message;
}

void AddBoundedObject(BMessage *object, const BMessage *bounded_by) {
	object->AddMessage("Bounded_by",bounded_by);
}


BMessage *CreateLight(double X, double Y, double Z, rgb_color &color) {
	BMessage *message = new BMessage(C_NEWLIGHT);
	message->AddInt32("Type",C_LIGHT);
	// Add the light information
	message->AddDouble("X",X);
	message->AddDouble("Y",Y);
	message->AddDouble("Z",Z);
	message->AddInt16("ColorRed",color.red);
	message->AddInt16("ColorGreen",color.green);
	message->AddInt16("ColorBlue",color.blue);
	return message;
}


void SetRange(BMessage *message,double min,double max) {
	message->AddDouble("RangeMin",min);
	message->AddDouble("RangeMax",max);
}

BMessage *CreateSpot(double PosX,double PosY, double PosZ, double LookX, double LookY, double LookZ, double RadiusMin, double RadiusMax, rgb_color &color) {
	BMessage *message = new BMessage(C_NEWLIGHT);
	message->AddInt32("Type",C_SPOT);
	// Add the light information
	message->AddDouble("PosX",PosX);
	message->AddDouble("PosY",PosY);
	message->AddDouble("PosZ",PosZ);
	message->AddDouble("LookX",LookX);
	message->AddDouble("LookY",LookY);
	message->AddDouble("LookZ",LookZ);
	message->AddDouble("RadiusMin",RadiusMin);
	message->AddDouble("RadiusMax",RadiusMax);
	message->AddInt16("ColorRed",color.red);
	message->AddInt16("ColorGreen",color.green);
	message->AddInt16("ColorBlue",color.blue);
	return message;
}


BMessage *CreateSun(double X, double Y, double Z, rgb_color &color) {
	BMessage *message = new BMessage(C_NEWLIGHT);
	message->AddInt32("Type",C_SUN);
	// Add the light information
	message->AddDouble("X",X);
	message->AddDouble("Y",Y);
	message->AddDouble("Z",Z);
	message->AddInt16("ColorRed",color.red);
	message->AddInt16("ColorGreen",color.green);
	message->AddInt16("ColorBlue",color.blue);
	return message;
}


// bmessage textures
BMessage *CreateBump(void) {
	BMessage *message = new BMessage(C_TEXTURE_BUMP);
	return message;
}

BMessage *CreateChecker(int16 red1, int16 green1, int16 blue1, int16 red2, int16 green2, int16 blue2) {
	BMessage *message = new BMessage(C_TEXTURE_CHECKER);
	message->AddInt16("ColorRed1",red1);
	message->AddInt16("ColorGreen1",green1);
	message->AddInt16("ColorBlue1",blue1);
	message->AddInt16("ColorRed2",red2);
	message->AddInt16("ColorGreen2",green2);
	message->AddInt16("ColorBlue2",blue2);
	return message;
}

BMessage *CreateMapping(const char *filename, bool once, int8 type) {
	BMessage *message = new BMessage(C_TEXTURE_MAPPING);
	message->AddString("File",filename);
	message->AddBool("Once",once);
	message->AddInt8("Type",type);
	return message;
}

void AddMappingFilter(BMessage *mapping, int16 red, int16 green, int16 blue) {
	mapping->AddInt16("ColorFilterRed",red);
	mapping->AddInt16("ColorFilterGreen",green);
	mapping->AddInt16("ColorFilterBlue",blue);
}

BMessage *CreateNoiseBump(void) {
	BMessage *message = new BMessage(C_TEXTURE_NOISE_BUMP);
	return message;
}

void AddNoiseBumpLevel(BMessage *noise, float level) {
	noise->AddFloat("Level",level);
}

void AddNoiseOctave(BMessage *noise, int8 octave) {
	noise->AddInt8("Octaves",octave);
}

BMessage *CreateNoiseColor(void) {
	BMessage *message = new BMessage(C_TEXTURE_NOISE_COLOR);
	return message;
}

void AddColorMap(BMessage *colormap,double limit, int16 red, int16 green, int16 blue) {
	colormap->AddInt16("ColorRed",red);
	colormap->AddInt16("ColorGreen",green);
	colormap->AddInt16("ColorBlue",blue);
	colormap->AddDouble("Limit",limit);
}

void AddTexture(BMessage *dest, const BMessage *texture) {
	dest->AddMessage("Texture",texture);
}


BMessage *CreateScale(double X, double Y, double Z) {
	BMessage *message = new BMessage(C_SCALE);
	// Add the scale information
	message->AddDouble("X",X);
	message->AddDouble("Y",Y);
	message->AddDouble("Z",Z);
	return message;
}

BMessage *CreateRotate(double X, double Y, double Z) {
	BMessage *message = new BMessage(C_ROTATE);
	// Add the rotate information
	message->AddDouble("X",X);
	message->AddDouble("Y",Y);
	message->AddDouble("Z",Z);
	return message;
}

BMessage *CreateTranslate(double X, double Y, double Z) {
	BMessage *message = new BMessage(C_TRANSLATE);
	// Add the translate information
	message->AddDouble("X",X);
	message->AddDouble("Y",Y);
	message->AddDouble("Z",Z);
	return message;
}

void AddTransformation(BMessage *dest, const BMessage *transf) {
	dest->AddMessage("Transformation",transf);
}

